<?php 
/**
 * Plugin Name: Protect Track ID (Matomo Plugin)
 * Plugin URI: http://plugins.matomo.org/ProtectTrackID
 * Description: Provides a option to protect idSite using hash instead default numeric
 * Author: Joubert RedRat
 * Author URI: https://github.com/joubertredrat/Matomo-ProtectTrackID
 * Version: 2.9.0
 */
?><?php

/**
 * Matomo - Open source web analytics
 *
 * @link https://matomo.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 *
 * @copyright (c) 2016 Joubert RedRat
 * @author Joubert RedRat <eu+matomo@redrat.com.br>
 * @license MIT
 * @category Matomo_Plugins
 * @package ProtectTrackID
 */

namespace Piwik\Plugins\ProtectTrackID;

use Piwik\Plugin;

 
if (defined( 'ABSPATH')
&& function_exists('add_action')) {
    $path = '/matomo/app/core/Plugin.php';
    if (defined('WP_PLUGIN_DIR') && WP_PLUGIN_DIR && file_exists(WP_PLUGIN_DIR . $path)) {
        require_once WP_PLUGIN_DIR . $path;
    } elseif (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR && file_exists(WPMU_PLUGIN_DIR . $path)) {
        require_once WPMU_PLUGIN_DIR . $path;
    } else {
        return;
    }
    add_action('plugins_loaded', function () {
        if (function_exists('matomo_add_plugin')) {
            matomo_add_plugin(__DIR__, __FILE__, true);
        }
    });
}

class ProtectTrackID extends Plugin
{
    public function registerEvents(): array
    {
        return [
            'Tracker.getJavascriptCode' => 'encodeIdJavaScript',
            'SitesManager.getImageTrackingCode' => 'encodeIdImage',
            'Tracker.Request.getIdSite' => 'decodeId'
        ];
    }

    public function encodeIdJavaScript(&$codeImpl, $parameters): void
    {
        $settings = PluginSettings::createFromSettings();
        if (!$settings->hasValidValues()) {
            return;
        }

        $hasher = new Hasher($settings);
        $codeImpl['idSite'] = $hasher->encode($codeImpl['idSite']);
    }

    public function encodeIdImage(&$matomoUrl, &$urlParams): void
    {
        $settings = PluginSettings::createFromSettings();
        if (!$settings->hasValidValues()) {
            return;
        }

        $hasher = new Hasher($settings);
        $urlParams['idsite'] = $hasher->encode($urlParams['idsite']);
    }

    public function decodeId(&$idSite, $params): void
    {
        $settings = PluginSettings::createFromSettings();
        if (!$settings->hasValidValues()) {
            return;
        }

        if (!$this->isValidHash($settings, $params['idsite'])) {
            return;
        }

        $hasher = new Hasher($settings);
        $idSite = $hasher->decode($params['idsite']);
    }

    public function isValidHash(PluginSettings $settings, string $hash): bool
    {
        $regex = sprintf(
            '/^(%s){%s}$/',
            implode('|', str_split($settings->base)),
            $settings->length
        );

        return (bool) preg_match($regex, $hash);
    }
}
